# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

import wallaby.FX as FX
import wallaby.FXUI as FXUI
from functools import partial
from wallaby.qt_combat import *


from wallaby.pf.peer.incrementalMultiViewer import *
from twisted.internet import defer

class MultiViewTableModel(QtCore.QAbstractTableModel):
    def __init__(self, widget, cols=None, *args, **ka):
        QtCore.QAbstractTableModel.__init__(self, None)

        self._widget = widget

        self._peer = IncrementalMultiViewer(*args, delegate=self, **ka)

        self._updateChangedDeferred = None
        self._maxRow = self._minRow = -1

        self._columns  = []
        self._types    = []
        self._typeArgs = []
        self._labels   = []
        self._cache    = []
        self._parent   = QtCore.QModelIndex()

        if cols == None: cols = []

        self._minRowHeight = 0
        self._minColumnWidth = 0

        self._loading = 0

        self._sizeHints = {}

        if 'minRowHeight' in ka: self._minRowHeight = ka['minRowHeight']
        if 'minColumnWidth' in ka: self._minColumnWidth = ka['minColumnWidth']
        if 'sizeHints' in ka: self._sizeHints = ka['sizeHints']

        if self._minRowHeight == None: self._minRowHeight = 0
        if self._minColumnWidth == None: self._minColumnWidth = 0
        if self._sizeHints == None: self._sizeHints = {}

        if isinstance(cols, list) and len(cols) > 0 and isinstance(cols[0], dict):
            cols.insert(0, {'path':'id'})
            self._columns = []
            self._labels = []
            self._types = []
            self._typeArgs = []
            for cfg in cols:
                if cfg is None: continue
                self._columns.append(cfg.get('path', None))
                self._labels.append(cfg.get('label', None))
                self._types.append(FX.convertType(cfg.get('type', None)))
                self._typeArgs.append(cfg.get('type-args', None))
        else: 
            cols.insert(0, 'id')
            self._columns, self._labels = FX.splitList(cols, ':')
            self._types, self._typeArgs, self._columns = FX.extractType(self._columns)

    def initialData(self):
        if self._widget: self._widget.initialData()

    def flags(self, index):
        defaultFlags = QtCore.QAbstractTableModel.flags(self, index)

        if index.isValid():
            return QtCore.Qt.ItemIsDragEnabled | QtCore.Qt.ItemIsDropEnabled | defaultFlags
        else:
            return QtCore.Qt.ItemIsDropEnabled | defaultFlags

    # def supportedDropActions(self):
    #     return QtCore.Qt.MoveAction
    # 
    # def supportedDragActions(self):
    #     return QtCore.Qt.MoveAction

    def destroyPeers(self, remove=False):
        self._peer.destroy(remove)
        self.__peer = None

    def doMultiSelect(self, *args, **ka):
        if not self._peer: return

        self._peer.doMultiSelect(*args, **ka)

    def selectRow(self, row):
        self._widget.selectRow(row)

    def doSelect(self, idx, tab):
        if not self._peer: return

        self._peer.doSelect(idx, tab)

    # def insertRows(self, row, count, parent):
    #     print "insertRows", row, count
    #     return True

    # def removeRows(self, row, count, parent):
    #     print "removeRows", row, count
    #     return True

    def sort(self, field, order):
        if not self._peer: return
        self._peer.sort(field, order)

    def hasChildren(self, index):
        return False

    def clearSelection(self):
        self._widget.clearSelection()

    def beginInsertCB(self, f, cnt):
        self.beginInsertRows(self._parent, f, f + cnt - 1)

        i = f
        for j in range(cnt):
            cols = []
            for k in range(len(self._columns)): cols.append({'cached': False, 'value': None, 'loading': False})

            self._cache.insert(i, {'cache': cols, 'loading': False})
            i = i + 1

    def resetSelectionCB(self):
        if self._widget: self._widget.resetSelection()

    def endInsertCB(self, f, cnt):
        self.emit(QtCore.SIGNAL('rowsInserted'), self._parent, f, cnt - 1)
        self.endInsertRows()

    def beginDeleteCB(self, f, t):
        self.beginRemoveRows(self._parent, f, t-1)

        for i in range(f, t):
            if f < len(self._cache):
                self._cache.pop(f)
            else:
                print "Try to remove out of bounds!", f, "of", len(self._cache) 

    def endDeleteCB(self, f, t):
        self.emit(QtCore.SIGNAL('rowsRemoved'), self._parent, f, t-1)
        self.endRemoveRows()

    def updateCB(self, row):
        if row < len(self._cache):
            self._cache[row]['loading'] = False
            for i in range(len(self._cache[row]['cache'])): self._cache[row]['cache'][i]['cached'] = False
            self.dataChanged.emit(self.index(row, 0), self.index(row, len(self._columns)-1))
                
    def parent(self, index):
        return self._parent

    def index(self, row, col, parent=QtCore.QModelIndex()):
        return self.createIndex(row, col)

    def rowCount(self, parent): 
        return len(self._cache)

    def columnCount(self, parent): 
        return len(self._columns)

    def getID(self, row):
        if len(self._rows) <= row:
            return "N/A"
        else:
            return self._rows[row].id

    def _empty(self):
        return None

    def reorderRows(self, row, oldIndex, newIndex):
        i1 = newIndex
        i2 = newIndex+1

        if newIndex < oldIndex:
            i1 -= 1
            i2 -= 1

        self._peer.reorder(row, i1, i2)

    # def setData(self, index, value, role=QtCore.Qt.EditRole):
    #     print "Setting Data", value.toString(), "at", index.row(), index.column()
    #     return True

    def data(self, index, role=QtCore.Qt.DisplayRole): 
        if not index.isValid(): return self._empty()

        row = index.row()
        col = index.column()

        if role == QtCore.Qt.SizeHintRole:
            if self._columns[col] in self._sizeHints and isinstance(self._sizeHints[self._columns[col]], dict):
                hint = self._sizeHints[self._columns[col]]
                w = self._minColumnWidth
                h = self._minRowHeight
                if 'width' in hint and hint['width'] is not None: w = int(hint['width'])
                if 'height' in hint and hint['height'] is not None: h = int(hint['height'])
                # print w, h, col
                return QtCore.QSize(w, h)
            else:
                size = QtCore.QSize(self._minColumnWidth, self._minRowHeight)

            if len(self._columns) <= col or self.rowCount(self.parent) <= row or not self._cache[row]['cache'][col]['cached']:
                return size

            # if row == 0:
            #     print "Size", row, col, size.width(), size.height()

            val = self._cache[row]['cache'][col]['value']

            if isinstance(val, QtGui.QPixmap):
                size.setWidth(min(100, max(val.width()+1, size.width())))
                size.setHeight(min(100, max(val.height(), size.height())))

            elif isinstance(val, (list, tuple)):
                w = h = 0
                for e in val:
                    if isinstance(e, QtGui.QPixmap):
                        w += e.width()
                        if h < e.height(): h = e.height()

                size.setWidth(max(w+1, size.width()))
                size.setHeight(max(h, size.height()))
            else:
                return self._empty() 
            return size
 
        if role != QtCore.Qt.DisplayRole or self.rowCount(self._parent) <= row or len(self._columns) <= col: return self._empty()

        # if self._cache[row]['loading']:
        #     if col == 0:
        #         return "Loading..."
        #     else:
        #         return ""

        if self._cache[row]['cache'][col]['cached']:
            val = self._cache[row]['cache'][col]['value']
            if val == None:
                return '-'
            else:
                return val

        if self._cache[row]['cache'][col]['loading']:
            if col == 1:
                return "Loading..."
            else:
                return ""

        if not self._peer: return '-'

        self._cache[row]['cache'][col]['loading'] = True

        self._cache[row]['loading'] = True
        self._loading += 1
        d = self._peer.deferredGetValue(row, self._columns[col])
        d.addCallback(partial(self._updateField, row, col))

        if 'value' in self._cache[row]['cache'][col]:
            val = self._cache[row]['cache'][col]['value']
            if val is None: return ""
            else: return val
        else:
            if col == 1:
                return "Loading..."
            else:
                return ""

    def _updateField(self, row, col, val):
        self._loading -= 1
        if row >= len(self._cache): return

        self._cache[row]['cache'][col]['loading'] = False

        # if isinstance(val, QtGui.QPixmap):
        #     print val, val.width(), val.height()

        if val != None and self._types[col] == "image" and not isinstance(val, QtGui.QPixmap):
            if self._typeArgs[col] and len(self._typeArgs[col]) > 0:
                d = self._peer.getImage(row, self._typeArgs[col], documentID=val)
            else:
                d = self._peer.getImage(row, val)
            d.addCallback(partial(self._updateField, row, col))

        val = FXUI.renderType(val, self._types[col], self._typeArgs[col]) 

        self._cache[row]['loading'] = False
        self._cache[row]['cache'][col]['cached'] = True
        self._cache[row]['cache'][col]['value']  = val

        if self._minRow == -1 or row < self._minRow: self._minRow = row
        if self._maxRow == -1 or row > self._maxRow: self._maxRow = row

        if self._updateChangedDeferred is not None:
            try:
                self._updateChangedDeferred.cancel()
            except: pass

        from twisted.internet import reactor
        self._updateChangedDeferred = reactor.callLater(1.0, self._updateChanged)
        # self.dataChanged.emit(self.index(row, col), self.index(row, col))
        # if self._loading == 0: self.layoutChanged.emit()
        return val

    def _updateChanged(self):
        self._updateChangedDeferred = None
        self.dataChanged.emit(self.index(self._minRow, 0), self.index(self._maxRow, len(self._columns)-1))
        self._maxRow = self._minRow = -1

    def headerData(self, col, orientation, role):
        if not QtCore: return
        if orientation == QtCore.Qt.Horizontal and role == QtCore.Qt.DisplayRole:
            if col >= len(self._labels):
                return None

            return self._labels[col]

        if orientation == QtCore.Qt.Vertical and role == QtCore.Qt.DisplayRole:
            return '%03d' % (col+1)

        return None




# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

from functools import wraps, partial
#from sets import Set

import wallaby.FX as FX

from wallaby.qt_combat import *

class APIProperty(object):
    def __init__(self, type, extended=False, doc=u"", example=u"", default=None, readOnly=False):
        self.type = type
        self.extended = extended
        self.doc = doc
        self.example = example
        self.default = default
        self.readOnly = readOnly

try:
    MetaQObject = type(QtCore.QObject)

    def setObjectName(self, name, base=None):
        if base is not None: base.setObjectName(self, name)
        from wallaby.frontends.qt.widgets.baseWidget import BaseWidget
        BaseWidget.registerConfigViewer(self)
    
    def repartial(func, *parameters, **kparms):
        @wraps(func)
        def wrapped(self, *args, **kw):
            kw.update(kparms)
            return func(self, *(args + parameters), **kw)
        return wrapped
    
    class QWallabyMeta(MetaQObject):
        @staticmethod
        def _recursivePropertyDiscovery(cls=None, bases=None, dict=None):
            if cls != None:
                _bases = cls.__bases__
                _dict = cls.__dict__
            else:
                _bases = bases
                _dict = dict

            properties = {}

            for base in _bases:
                dict = QWallabyMeta._recursivePropertyDiscovery(cls=base)
                for k,v in dict.iteritems():
                    properties[k]=v

            for k,v in _dict.iteritems():
                if isinstance(v, property):
                    properties[k]=(v,_dict['wallabyProperties'][k])

            return properties

        def __new__(mcs, name, bases, dict):
            from wallaby.frontends.qt.widgets.baseWidget import BaseWidget
            newdict = {}
            properties = {}

            properties = QWallabyMeta._recursivePropertyDiscovery(bases=bases,dict=dict)
            for k,(v,p) in properties.iteritems():
                dict[k]=v
                properties[k]=p

            dict["wallabyType"] = APIProperty("string")

            for k in dict:
                if isinstance(dict[k], APIProperty):
                    prop = dict[k]
                    var = k[0:1].upper() + k[1:]
                    properties[k] = prop
    
                    newdict['set' + var] = repartial(BaseWidget.setConfig, key=k, type=prop.type)
                    newdict['get' + var] = repartial(BaseWidget.getConfig, key=k, type=prop.type)
                    newdict['reset' + var] = repartial(BaseWidget.resetConfig, key=k, type=prop.type)

                    newdict[k] = property(fget=newdict['get' + var], fset=newdict['set' + var])

            dict["template"] = Property("QString", fget   = repartial(BaseWidget.getWallabyTemplate),
                                                   fset   = repartial(BaseWidget.setWallabyTemplate),
                                                   freset = repartial(BaseWidget.resetWallabyTemplate))

            dict['wallabyProperties'] = properties 
    
            dict['setObjectName'] = repartial(setObjectName, base=bases[0])

            for k, v in newdict.items():
                dict[k] = v

            cls = MetaQObject.__new__(mcs, name, bases, dict)
            return cls
except:
    pass

class Meta(object):
    @staticmethod
    def property(type, extended=False, doc=u"", example=u"", default=None, readOnly=False):
       return APIProperty(type, extended=extended, doc=doc, example=example, default=default, readOnly=readOnly)



# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.

# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/dominik/Documents/Projekte/wallaby/wallaby-frontend-qt/wallaby/frontends/qt/loginDialog.ui'
#
# Created: Mon Nov 19 14:05:03 2012
#      by: PyQt4 UI code generator 4.9.4
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_LoginDialog(object):
    def setupUi(self, LoginDialog):
        LoginDialog.setObjectName(_fromUtf8("LoginDialog"))
        LoginDialog.setWindowModality(QtCore.Qt.ApplicationModal)
        LoginDialog.resize(390, 267)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(LoginDialog.sizePolicy().hasHeightForWidth())
        LoginDialog.setSizePolicy(sizePolicy)
        LoginDialog.setModal(True)
        self.verticalLayout = QtGui.QVBoxLayout(LoginDialog)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.groupBox = QtGui.QGroupBox(LoginDialog)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.userEdit = QtGui.QLineEdit(self.groupBox)
        self.userEdit.setGeometry(QtCore.QRect(170, 100, 161, 22))
        self.userEdit.setText(_fromUtf8(""))
        self.userEdit.setObjectName(_fromUtf8("userEdit"))
        self.pwdEdit = QtGui.QLineEdit(self.groupBox)
        self.pwdEdit.setGeometry(QtCore.QRect(170, 140, 161, 22))
        self.pwdEdit.setEchoMode(QtGui.QLineEdit.Password)
        self.pwdEdit.setObjectName(_fromUtf8("pwdEdit"))
        self.image = QtGui.QLabel(self.groupBox)
        self.image.setGeometry(QtCore.QRect(20, 40, 131, 141))
        self.image.setText(_fromUtf8(""))
        self.image.setPixmap(QtGui.QPixmap(_fromUtf8(":/images/images/user_big.png")))
        self.image.setObjectName(_fromUtf8("image"))
        self.verticalLayout.addWidget(self.groupBox)
        self.buttonBox = QtGui.QDialogButtonBox(LoginDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName(_fromUtf8("buttonBox"))
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(LoginDialog)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL(_fromUtf8("accepted()")), LoginDialog.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL(_fromUtf8("rejected()")), LoginDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(LoginDialog)

    def retranslateUi(self, LoginDialog):
        LoginDialog.setWindowTitle(QtGui.QApplication.translate("LoginDialog", "Dialog", None, QtGui.QApplication.UnicodeUTF8))
        self.groupBox.setTitle(QtGui.QApplication.translate("LoginDialog", "Anmeldung", None, QtGui.QApplication.UnicodeUTF8))
        self.userEdit.setPlaceholderText(QtGui.QApplication.translate("LoginDialog", "Benutzername", None, QtGui.QApplication.UnicodeUTF8))
        self.pwdEdit.setPlaceholderText(QtGui.QApplication.translate("LoginDialog", "Passwort", None, QtGui.QApplication.UnicodeUTF8))

import resource_rc



#!/usr/bin/env python
# Copyright (c) by it's authors. 
# Some rights reserved. See LICENSE, AUTHORS.


import wallaby.FX as FX
import wallaby.FXUI as FXUI

from wallaby.qt_combat import *

import wallaby.frontends.qt.resource_rc as resource_rc
from wallaby.frontends.qt.loginDialog import *
import twisted.application.service
import sys, time

from twisted.internet import defer

import wallaby.frontends.qt.reactor.threadedselect as threadedselect

from wallaby.frontends.qt.baseqobject import *
from wallaby.common.fxLogger import *

from wallaby.pf.peer import *
from wallaby.pf.peer.debugger import *
from wallaby.pf.room import *

import signal
import wallaby.frontends.qt.resource_rc

import shutil

class WallabyApp(object):
    def __init__(self, appName = 'example', checkRoom = None, suggest = False, options=None):
        splash = None

        FXUI.app = QtGui.QApplication(sys.argv)
        FXUI.app.setApplicationName("wallaby - " + appName)

        for s in ['16', '32', '64', '128', '256']:
            FXUI.app.setWindowIcon(QtGui.QIcon(QtGui.QPixmap(':/icons/images/wallaby_logo_' + s + '.png')))

        pixmap = QtGui.QPixmap(":/images/images/wallaby_splash.png")
        splash = QtGui.QSplashScreen(pixmap)
        splash.show()
        splash.raise_()
        FXUI.app.processEvents()

        if USES_PYSIDE:
            import wallaby.frontends.qt.reactor.qtreactor as qtreactor
            qtreactor.install()
        else:
            threadedselect.install()
             
            from twisted.internet import reactor
            ii = Interleaver()
            reactor.interleave(ii.toInterleave)
            reactor.suggestThreadPoolSize(50)

        FXUI.mineIcon = QtGui.QIcon(':/icons/images/mine.png')
        FXUI.theirsIcon = QtGui.QIcon(':/icons/images/theirs.png')

        tapp = twisted.application.service.Application("gui")
        service  = FXLogger('wallaby.log')
        service.setServiceParent(tapp)
        service.startService()

        FX.appModule = 'wallaby.apps.' + appName

        try:
            from twisted.plugin import getCache
            pkg = __import__(FX.appModule, globals(), locals(), ["*"], 0)
            if pkg is not None and len(pkg.__path__) > 0 and os.path.exists(pkg.__path__[0]):
                FX.appPath = pkg.__path__[0]
            else:
                FX.appPath = os.path.join(".", "wallaby", "apps", appName)
        except:
            FX.appPath = os.path.join(".", "wallaby", "apps", appName)

        try:
            print "importing", FX.appModule
            mod = FX.imp(FX.appModule + '.mainWindow', False)
        except:
            mod = None

        if mod == None:
            FX.crit('Module', FX.appModule, 'not found')
            reactor.callWhenRunning(self.myQuit)
            if USES_PYSIDE: reactor.runReturn()
            FXUI.app.exec_()
            return

        try:
            FXUI.mainWindow = mod.MainWindow(self.myQuit, options)
        except Exception as e:
            import traceback
            traceback.print_exc(file=sys.stdout)

            from twisted.internet import reactor
            reactor.callWhenRunning(self.myQuit)
            if USES_PYSIDE: reactor.runReturn()
            FXUI.app.exec_()
            return

        FXUI.mainWindow.setSplash(splash)

        from twisted.internet import reactor
        reactor.callWhenRunning(self.run, mod, options, checkRoom)

        FXUI.mainWindow.enabled = False
        FXUI.mainWindow.configure()
        FXUI.mainWindow.show()
        FXUI.mainWindow.raise_()

        signal.signal(signal.SIGINT, self.sigint_handler)
        signal.signal(signal.SIGTERM, self.sigint_handler)

        # self.gc = GarbageCollector(FXUI.mainWindow, True)

        if USES_PYSIDE: reactor.runReturn()
        FXUI.app.exec_()

    @defer.inlineCallbacks
    def run(self, mod, options, checkRoom): 
        if options is not None:
            authenticated = yield FXUI.mainWindow.authenticated(options.username, options.password, options)
        else:
            authenticated = yield FXUI.mainWindow.authenticated()

        count = 0
        while not authenticated:
            if count == 3: self.realQuit(0)

            dlg = LoginDialog()
            code = dlg.exec_()

            if code == 0: self.realQuit(0)

            try:
                authenticated = yield FXUI.mainWindow.authenticated(unicode(dlg.userEdit.text()), unicode(dlg.pwdEdit.text()), options)
            except:
                return

            count += 1

        from twisted.internet import reactor

        if options is not None: FXUI.mainWindow.setDebuggedRooms(options.debug)
        FXUI.mainWindow.run(checkRoom)
        FXUI.mainWindow.enabled = True

    def realQuit(self, shuttime):
        if FXUI.app != None:
            if shuttime > 0:
                print "Shutdown in ", (shuttime-1), "seconds"
                from twisted.internet import reactor
                reactor.callLater(1, self.realQuit, shuttime-1)
                return

            from wallaby.frontends.qt.widgets.baseWidget import BaseWidget
            if FXUI.mainWindow is not None:
                for w in FXUI.mainWindow.findChildren(BaseWidget):
                    w.deregister(True)

            app = FXUI.app
            del FXUI.app
            del FXUI.mainWindow

            print "Stopping Reactor"
            from twisted.internet import reactor
            reactor.stop()

            FX.info("Stopping app")
            print "Stopping App"

            app.exit()
            # FXUI.app.quit()
            FX.info("Stopping reactor")
            FX.info("Bye")

            app = None

    def myQuit(self):
        print "Set shutdown flag"
        FX.shutdown = True
        FX.callShutdownCBs()
        shuttime = 0
        from twisted.internet import reactor
        print "Shutdown in ", shuttime, "seconds"
        reactor.callLater(1, self.realQuit, shuttime)

    def sigint_handler(self, *args):
        """Handler for the SIGINT signal."""
        self.myQuit()
